function [varargout] = costfunc(varargin)

% Information:
% ============
% Written by: Mikael Sunnaker, mikael.sunnaker@gmail.com
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
% USA.

persistent dataInfo files parStr nrOutputs nrStates ipar

if(nargin == 3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initialize the objective function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     %set a random seed
%     s = RandStream('mcg16807','Seed','shuffle');
%     RandStream.setDefaultStream(s);

    %Only first function call, but not in the estimation
    %get the function arguments
    model = varargin{1};
    dataInfo = varargin{2};
    ipar = varargin{3};
    %get the number of states in the model
    nrStates = length(SBstates(model));    
    
    %Define the Jacobians and functions
    Asym = SBPDsymjacobian(model,1); %second argument simplified or not
    files.Asym = Asym;
    
    %create the alternative mex version of the model
    model = addPS(model);
    %[ODEfctname,ODEfilefullpath,DATAfctname,EVENTfctname,EVENTASSGNfctname] = TAcreateTempODEfile(model,1,0,Asym);
    %SBPDmakeMEXmodelAlt(model,'mexModel',0,Asym.jacobian);
    %files.ODEfctname = ODEfctname;
    files.model = model;
    
    %output/response function, y
    nrOutputs = size(dataInfo.data,2); 
    %the C matrix (under the assumption that all states are measured)
    files.Cmat = eye(nrOutputs); %dy/dx
    
    %specify the parameters
    parStr.ic = SBinitialconditions(model);
    [parStr.parNames,parStr.parameters] = SBparameters(model);
    parStr.nrParameters = length(parStr.parNames);
    parStr.P = 1*eye(nrStates); %the states covariance matrix
    parStr.S = 1*eye(nrOutputs); %the output covariance matrix 
    
    %define the function output
    varargout{1} = parStr;
    varargout{2} = dataInfo;
    
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Parameter estimation part
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %get the parameters values that should be evaluated
    testPar = 10.^(varargin{1});
    
    try
        %structural parameters and sigma parameters
        parStr.parameters(ipar) = testPar;                
         
        %change k2(=1) to test evaluating an incorrect model structure...
        %parStr.parameters(2) = 10;   
        
        %Specify P0, P are now states in the model
        parStr.P = 0 * eye(nrStates);
        %Specify the output covariance matrix S
        parStr.S = 0.1 * eye(nrOutputs);
        
        %run the extended Kalman filter
        [eps,Rpred] = EKF(files,parStr,dataInfo);
        
        %evaluate the likelihood function
        varargout{1} = -logLikelihoodFunction(eps,Rpred,length(dataInfo.tp));
        
    catch
        varargout{1} = inf;
    end
end

return

