%Topological augmentation script

% Information:
% ============
% Written by: Mikael Sunnaker, mikael.sunnaker@gmail.com
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
% USA.

clear
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Model definition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%model 2
model = SBmodel('model2.txt');
ipar = find([0 0 0 1 0 0 1]); %k and sigma1
vp = log10([1 1]);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate experimental data for the model 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datares = SBsimulate(model,0:0.5:10);
%define the experimental time points
dataInfo.tp = datares.time;
%define the experimental data
dataInfo.data = datares.statevalues + 0.1*randn(size(datares.statevalues));

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Total
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%number of parameters
dim = length(vp);

%initialize the objective function
costfunc(model,dataInfo,ipar);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%evaluate combinations of parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%define the parameter values to be evaluated
pvalv = [0.1 0.25 0.5 1 4 10];
nrP = length(pvalv);
vpvect = zeros(nrP);
count = 0;
for i = 1:nrP
    for k = 1:nrP
        count = count + 1
        %choose the parameter values
        p1 = pvalv(i);
        p2 = pvalv(k); 
        %choose the parameter point
        vp = log10([p1 p2]);
        %evalute the starting point
        vpvect(i,k) = costfunc(vp);
    end
end

%%
%plot the (quasi) log-likelihood function
figure(1)
%define a grid for the evaluations
ppoints = 1:length(pvalv);
[Xx,Yy] = meshgrid(pvalv(ppoints));
surf(Xx,Yy,vpvect(ppoints,ppoints))
xlabel('\sigma_2')
ylabel('\sigma_1')
zlabel('-log(L)')

%plot experimental data
figure(2)
plot(dataInfo.tp,dataInfo.data,'.:')
xlabel('time')
ylabel('concentration')
legend('A','B')

%save the results in a mat file 
dfname = 'datafile2';
save(dfname,'vpvect','Xx','Yy','pvalv','dataInfo')

